import os

import pytest
from d8s_file_system import directory_create, directory_delete, file_write

from d8s_archives import archive_create, archive_read

from .test_archive_utils import subprocess_run

TEST_DIRECTORY_PATH = './_test_data/archive/'
TEST_DIRECTORY_BASE_PATH = './_test_data/'
DEFAULT_FILE_CONTENTS = 'abc'


@pytest.fixture(autouse=True)
def clear_testing_directory():
    """This function is run after every test."""
    directory_delete(TEST_DIRECTORY_PATH)
    directory_create(TEST_DIRECTORY_PATH)


def setup_module():
    """This function is run before all of the tests in this file are run."""
    directory_create(TEST_DIRECTORY_PATH)


def teardown_module():
    """This function is run after all of the tests in this file are run."""
    directory_delete(TEST_DIRECTORY_BASE_PATH)


def _create_sample_file(path, contents=DEFAULT_FILE_CONTENTS):
    full_path = os.path.join(TEST_DIRECTORY_PATH, path)
    result = file_write(full_path, contents)
    assert result
    return full_path


def test_archive_read():
    file_path = _create_sample_file('a')
    output_dir = '_test_data/archive/a.zip'

    archive_create(file_path, output_dir)
    assert list(archive_read(output_dir)) == [('a', DEFAULT_FILE_CONTENTS)]
    assert list(archive_read(output_dir, archive_name='a')) == [('a', DEFAULT_FILE_CONTENTS)]


def test_archive_read_password_encrypted():
    password = 'foo'

    file_path = _create_sample_file('a')
    zip_path = os.path.join(TEST_DIRECTORY_PATH, 'a.zip')
    command = f'zip -P foo {zip_path} {file_path}'
    subprocess_run(command, input_=password)

    results = list(archive_read(zip_path, password=password))
    assert results == [('_test_data/archive/a', 'abc')]



import subprocess


def subprocess_run(command, input_=None):
    """Run the given command as if it were run in a command line."""
    if isinstance(command, str):
        command_list = command.split(' ')
    else:
        command_list = command

    process = subprocess.run(command_list, input=input_, text=True, capture_output=True)
    result = (process.stdout, process.stderr)
    return result



from typing import Iterable, Tuple


def _archive_zip(output_path):
    """Return the file at the output_path as a zipfile."""
    import zipfile

    return zipfile.ZipFile(output_path, mode='w')


def archive_create(file_path, output_path, *, archive_name=None):
    """Archive the given file."""
    from d8s_file_system import file_name

    if archive_name is None:
        archive_name = file_name(file_path)

    with _archive_zip(output_path) as zipped_file:
        zipped_file.write(file_path, arcname=archive_name)


def _archive_unzip(archive_path):
    """Read the file at the archive_path as a zipfile."""
    import zipfile

    return zipfile.ZipFile(archive_path)


# TODO: I think the `archive_name` parameter should be changed to something like `file_name`
# TODO: I think the `file_path` parameter should be changed to something like `archive_path`
# TODO: we should probably allow an encoding to be passed in for decoding the zip contents
def archive_read(file_path, *, archive_name=None, password=None) -> Iterable[Tuple[str, str]]:
    """Read file(s) from the archive. If archive_name is given, read only that file; otherwise, read all files."""
    from d8s_strings import bytes_decode_as_string, string_encode_as_bytes

    if password:
        password = string_encode_as_bytes(password)

    with _archive_unzip(file_path) as unzipped_archive:
        if archive_name:
            yield archive_name, bytes_decode_as_string(unzipped_archive.read(archive_name, pwd=password))
        else:
            for file_name in unzipped_archive.namelist():
                yield file_name, bytes_decode_as_string(unzipped_archive.read(file_name, pwd=password))


# TODO: make a function out of this call: unzipped_archive.namelist()
